@@ -37,6 +37,10 @@
 #include "vino-enums.h"
 #include <sys/poll.h>
 
+#ifdef VINO_ENABLE_KEYRING
+#include <gnome-keyring.h>
+#endif
+
 /* If an authentication attempt failes, delay the next
  * authentication attempt for 5 seconds.
  */
@@ -63,11 +67,15 @@ struct _VinoServerPrivate
   VinoHTTP         *http;
 #endif
 
+  int               alternative_port;
+
   guint             on_hold : 1;
+  guint             local_only : 1;
   guint             prompt_enabled : 1;
   guint             view_only : 1;
   guint             require_encryption : 1;
   guint             last_auth_failed : 1;
+  guint             use_alternative_port : 1;
 };
 
 typedef struct
@@ -90,6 +98,9 @@ enum
   PROP_ON_HOLD,
   PROP_PROMPT_ENABLED,
   PROP_VIEW_ONLY,
+  PROP_LOCAL_ONLY,
+  PROP_USE_ALTERNATIVE_PORT,
+  PROP_ALTERNATIVE_PORT,
   PROP_REQUIRE_ENCRYPTION,
   PROP_AUTH_METHODS,
   PROP_VNC_PASSWORD
@@ -100,8 +111,9 @@ static enum rfbNewClientAction vino_server_auth_client (VinoServer           *se
 							const char           *response,
 							int                   length);
 
-static void vino_server_setup_framebuffer   (VinoServer *server);
-static void vino_server_release_framebuffer (VinoServer *server);
+static void vino_server_setup_framebuffer     (VinoServer *server);
+static void vino_server_release_framebuffer   (VinoServer *server);
+static void vino_server_update_security_types (VinoServer *server);
 
 static gpointer parent_class;
 
@@ -139,7 +151,8 @@ vino_server_handle_client_gone (rfbClientPtr rfb_client)
 	    g_source_remove (client->update_timeout);
 	  client->update_timeout = 0;
 
-	  g_source_remove (client->io_watch);
+          if (client->io_watch)
+            g_source_remove (client->io_watch);
 	  client->io_watch = 0;
 
 	  g_io_channel_unref (client->io_channel);
@@ -240,7 +253,7 @@ vino_server_set_client_on_hold (VinoServer            *server,
     {
       if (client->io_watch)
         {
-          g_source_remove(client->io_watch);
+          g_source_remove (client->io_watch);
           client->io_watch = 0;
         }
 
@@ -263,8 +276,8 @@ vino_server_set_client_on_hold (VinoServer            *server,
       if (!client->update_timeout)
         {
           client->update_timeout = g_timeout_add (50,
-					(GSourceFunc) vino_server_update_client_timeout,
-					rfb_client);
+                                                  (GSourceFunc) vino_server_update_client_timeout,
+                                                  rfb_client);
         }
     }
 }
@@ -439,6 +452,44 @@ vino_server_defer_client_auth (VinoServer           *server,
 					client);
 }
 
+static char *
+vino_server_get_password_from_keyring (VinoServer *server)
+{
+#ifdef VINO_ENABLE_KEYRING
+  GnomeKeyringNetworkPasswordData *found_item;
+  GnomeKeyringResult               result;
+  GList                           *matches;
+  char                            *password;
+  
+  matches = NULL;
+
+  result = gnome_keyring_find_network_password_sync (
+                NULL,           /* user     */
+		NULL,           /* domain   */
+		"vino.local",   /* server   */
+		NULL,           /* object   */
+		"rfb",          /* protocol */
+		"vnc-password", /* authtype */
+		5900,           /* port     */
+		&matches);
+
+  if (result != GNOME_KEYRING_RESULT_OK)
+    return NULL;
+
+  g_assert (matches != NULL && matches->data != NULL);
+
+  found_item = (GnomeKeyringNetworkPasswordData *) matches->data;
+
+  password = g_strdup (found_item->password);
+
+  gnome_keyring_network_password_list_free (matches);
+
+  return password;
+#else
+  return NULL;
+#endif
+}
+
 static enum rfbNewClientAction
 vino_server_auth_client (VinoServer           *server,
 			 VinoServerClientInfo *client,
@@ -451,13 +502,20 @@ vino_server_auth_client (VinoServer           *server,
   if (!(server->priv->auth_methods & VINO_AUTH_VNC))
     goto auth_failed;
 
-  if (!server->priv->vnc_password)
-    goto auth_failed;
-
-  if (!(password = vino_base64_unencode (server->priv->vnc_password)))
+  if (!(password = vino_server_get_password_from_keyring (server)))
     {
-      g_warning ("Failed to base64 unencode VNC password\n");
-      goto auth_failed;
+      guchar *blob;
+      gsize   blob_len;
+
+      if (!server->priv->vnc_password)
+        goto auth_failed;
+
+      blob_len = 0;
+      blob = g_base64_decode (server->priv->vnc_password, &blob_len);
+
+      password = g_strndup ((char *) blob, blob_len);
+
+      g_free (blob);
     }
 
   rfb_client = client->rfb_client;
@@ -668,9 +726,17 @@ vino_server_init_from_screen (VinoServer *server,
    *   5900-6000
    */
   rfb_screen->rfbDeferUpdateTime = 0;
+  rfb_screen->localOnly          = server->priv->local_only;
   rfb_screen->autoPort           = TRUE;
+  rfb_screen->rfbPort            = VINO_SERVER_DEFAULT_PORT;
   rfb_screen->rfbAlwaysShared    = TRUE;
 
+  if (server->priv->use_alternative_port)
+    {
+      rfb_screen->autoPort = FALSE;
+      rfb_screen->rfbPort  = server->priv->alternative_port;
+    }
+
   rfbInitServer (rfb_screen);
   
   /* libvncserver NOTE:
@@ -691,6 +757,8 @@ vino_server_init_from_screen (VinoServer *server,
 
   rfb_screen->passwordCheck = vino_server_check_vnc_password;
 
+  vino_server_update_security_types (server);
+
   dprintf (RFB, "Creating watch for listening socket %d - port %d\n",
 	   rfb_screen->rfbListenSock, rfb_screen->rfbPort);
 
@@ -785,6 +853,9 @@ vino_server_set_property (GObject      *object,
     case PROP_VIEW_ONLY:
       vino_server_set_view_only (server, g_value_get_boolean (value));
       break;
+    case PROP_LOCAL_ONLY:
+      vino_server_set_local_only (server, g_value_get_boolean (value));
+      break;
     case PROP_REQUIRE_ENCRYPTION:
       vino_server_set_require_encryption (server, g_value_get_boolean (value));
       break;
@@ -794,6 +865,12 @@ vino_server_set_property (GObject      *object,
     case PROP_VNC_PASSWORD:
       vino_server_set_vnc_password (server, g_value_get_string (value));
       break;
+    case PROP_USE_ALTERNATIVE_PORT:
+      vino_server_set_use_alternative_port (server, g_value_get_boolean (value));
+      break;
+    case PROP_ALTERNATIVE_PORT:
+      vino_server_set_alternative_port (server, g_value_get_int (value));
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -822,8 +899,11 @@ vino_server_get_property (GObject    *object,
     case PROP_VIEW_ONLY:
       g_value_set_boolean (value, server->priv->view_only);
       break;
+    case PROP_LOCAL_ONLY:
+      g_value_set_boolean (value, server->priv->local_only);
+      break;
     case PROP_REQUIRE_ENCRYPTION:
-      g_value_set_boolean (value, server->priv->view_only);
+      g_value_set_boolean (value, server->priv->require_encryption);
       break;
     case PROP_AUTH_METHODS:
       g_value_set_flags (value, server->priv->auth_methods);
@@ -831,6 +911,12 @@ vino_server_get_property (GObject    *object,
     case PROP_VNC_PASSWORD:
       g_value_set_string (value, server->priv->vnc_password);
       break;
+    case PROP_USE_ALTERNATIVE_PORT:
+      g_value_set_boolean (value, server->priv->use_alternative_port);
+      break;
+    case PROP_ALTERNATIVE_PORT:
+      g_value_set_int (value, server->priv->alternative_port);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -886,6 +972,14 @@ vino_server_class_init (VinoServerClass *klass)
 							 FALSE,
 							 G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
 
+  g_object_class_install_property (gobject_class,
+				   PROP_LOCAL_ONLY,
+				   g_param_spec_boolean ("local-only",
+							 _("Local Only"),
+							 _("Only allow local connections"),
+							 FALSE,
+							 G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
+
   g_object_class_install_property (gobject_class,
 				   PROP_REQUIRE_ENCRYPTION,
 				   g_param_spec_boolean ("require-encryption",
@@ -910,6 +1004,24 @@ vino_server_class_init (VinoServerClass *klass)
 							_("The password (base64 encoded) used to authenticate types using the VncAuth method"),
 							NULL,
 							G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
+
+  g_object_class_install_property (gobject_class,
+				   PROP_USE_ALTERNATIVE_PORT,
+				   g_param_spec_boolean ("use-alternative-port",
+							 _("Use an alternative port"),
+							 _("Listen on the port specified by the 'alternative-port' property"),
+							 FALSE,
+							 G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
+
+  g_object_class_install_property (gobject_class,
+				   PROP_ALTERNATIVE_PORT,
+				   g_param_spec_int ("alternative-port",
+						     _("Alternative port number"),
+						     _("Listen on the specified port number if the 'use-alternative-port' property is TRUE"),
+						     VINO_SERVER_MIN_PORT,
+						     VINO_SERVER_MAX_PORT,
+						     VINO_SERVER_DEFAULT_PORT,
+						     G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
 }
 
 GType
@@ -985,6 +1097,94 @@ vino_server_set_view_only (VinoServer *server,
     }
 }
 
+gboolean
+vino_server_get_local_only (VinoServer *server)
+{
+  g_return_val_if_fail (VINO_IS_SERVER (server), FALSE);
+
+  return server->priv->local_only;
+}
+
+void
+vino_server_set_local_only (VinoServer *server,
+                            gboolean    local_only)
+{
+  g_return_if_fail (VINO_IS_SERVER (server));
+
+  local_only = local_only != FALSE;
+
+  if (server->priv->local_only != local_only)
+    {
+      server->priv->local_only = local_only;
+
+      if (server->priv->rfb_screen != NULL)
+        rfbSetLocalOnly (server->priv->rfb_screen,
+                         server->priv->local_only);
+
+      g_object_notify (G_OBJECT (server), "local-only");
+    }
+}
+
+gboolean
+vino_server_get_use_alternative_port (VinoServer *server)
+{
+  g_return_val_if_fail (VINO_IS_SERVER (server), FALSE);
+
+  return server->priv->use_alternative_port;
+}
+
+void
+vino_server_set_use_alternative_port (VinoServer *server,
+                                      gboolean    use_alternative_port)
+{
+  g_return_if_fail (VINO_IS_SERVER (server));
+
+  use_alternative_port = use_alternative_port != FALSE;
+
+  if (server->priv->use_alternative_port != use_alternative_port)
+    {
+      server->priv->use_alternative_port = use_alternative_port;
+
+      if (server->priv->rfb_screen)
+        {
+          if (server->priv->use_alternative_port)
+            rfbSetPort (server->priv->rfb_screen,
+                        server->priv->alternative_port);
+
+          rfbSetAutoPort (server->priv->rfb_screen,
+                          !server->priv->use_alternative_port);
+        }
+
+      g_object_notify (G_OBJECT (server), "use-alternative-port");
+    }
+}
+
+gint
+vino_server_get_alternative_port (VinoServer *server)
+{
+  g_return_val_if_fail (VINO_IS_SERVER (server), FALSE);
+
+  return server->priv->alternative_port;
+}
+
+void
+vino_server_set_alternative_port (VinoServer *server,
+                                  int         alternative_port)
+{
+  g_return_if_fail (VINO_IS_SERVER (server));
+
+  if (server->priv->alternative_port != alternative_port)
+    {
+      server->priv->alternative_port = alternative_port;
+
+      if (server->priv->rfb_screen &&
+          server->priv->use_alternative_port)
+        rfbSetPort (server->priv->rfb_screen, server->priv->alternative_port);
+
+      g_object_notify (G_OBJECT (server), "alternative-port");
+    }
+}
+
 gboolean
 vino_server_get_on_hold (VinoServer *server)
 {
@@ -1054,6 +1254,9 @@ vino_server_set_prompt_enabled (VinoServer *server,
 static void
 vino_server_update_security_types (VinoServer *server)
 {
+  if (!server->priv->rfb_screen)
+    return;
+
   rfbClearSecurityTypes (server->priv->rfb_screen);
   rfbClearAuthTypes (server->priv->rfb_screen);
 
